@@ -1,6 +1,6 @@
 ;;; magit-commit.el --- create Git commits  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2008-2016  The Magit Project Contributors
+;; Copyright (C) 2008-2017  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -45,7 +45,7 @@
 
 (defcustom magit-commit-arguments nil
   "The arguments used when committing."
-  :group 'magit-commands
+  :group 'magit-git-arguments
   :type '(repeat (string :tag "Argument")))
 
 (defcustom magit-commit-ask-to-stage 'verbose
@@ -85,7 +85,7 @@ an error while using those is harder to recover from."
   :group 'magit-commands
   :type 'boolean)
 
-;;; Code
+;;; Popup
 
 (defun magit-commit-popup (&optional arg)
   "Popup console for commit commands."
@@ -125,13 +125,24 @@ an error while using those is harder to recover from."
       magit-current-popup-args
     magit-commit-arguments))
 
-(defun magit-commit-message-buffer ()
-  (let* ((find-file-visit-truename t) ; git uses truename of COMMIT_EDITMSG
-         (topdir (magit-toplevel)))
-    (--first (equal topdir (with-current-buffer it
-                             (and git-commit-mode (magit-toplevel))))
-             (append (buffer-list (selected-frame))
-                     (buffer-list)))))
+(defvar magit-gpg-secret-key-hist nil)
+
+(defun magit-read-gpg-secret-key (prompt &optional _initial-input)
+  (require 'epa)
+  (let ((keys (--map (concat (epg-sub-key-id (car (epg-key-sub-key-list it)))
+                             " "
+                             (-when-let (id-obj (car (epg-key-user-id-list it)))
+                               (let    ((id-str (epg-user-id-string id-obj)))
+                                 (if (stringp id-str)
+                                     id-str
+                                   (epg-decode-dn id-obj)))))
+                     (epg-list-keys (epg-make-context epa-protocol) nil t))))
+    (car (split-string (magit-completing-read
+                        prompt keys nil nil nil 'magit-gpg-secret-key-hist
+                        (car (or magit-gpg-secret-key-hist keys)))
+                       " "))))
+
+;;; Commands
 
 ;;;###autoload
 (defun magit-commit (&optional args)
@@ -302,6 +313,8 @@ depending on the value of option `magit-commit-squash-confirm'."
    (t
     (user-error "Nothing staged"))))
 
+;;; Pending Diff
+
 (defun magit-commit-diff ()
   (-when-let (fn (and git-commit-mode
                       magit-commit-show-diff
@@ -330,22 +343,15 @@ depending on the value of option `magit-commit-squash-confirm'."
 (add-to-list 'with-editor-server-window-alist
              (cons git-commit-filename-regexp 'switch-to-buffer))
 
-(defvar magit-gpg-secret-key-hist nil)
+;;; Message Utilities
 
-(defun magit-read-gpg-secret-key (prompt &optional _initial-input)
-  (require 'epa)
-  (let ((keys (--map (concat (epg-sub-key-id (car (epg-key-sub-key-list it)))
-                             " "
-                             (-when-let (id-obj (car (epg-key-user-id-list it)))
-                               (let    ((id-str (epg-user-id-string id-obj)))
-                                 (if (stringp id-str)
-                                     id-str
-                                   (epg-decode-dn id-obj)))))
-                     (epg-list-keys (epg-make-context epa-protocol) nil t))))
-    (car (split-string (magit-completing-read
-                        prompt keys nil nil nil 'magit-gpg-secret-key-hist
-                        (car (or magit-gpg-secret-key-hist keys)))
-                       " "))))
+(defun magit-commit-message-buffer ()
+  (let* ((find-file-visit-truename t) ; git uses truename of COMMIT_EDITMSG
+         (topdir (magit-toplevel)))
+    (--first (equal topdir (with-current-buffer it
+                             (and git-commit-mode (magit-toplevel))))
+             (append (buffer-list (selected-frame))
+                     (buffer-list)))))
 
 (defvar magit-commit-add-log-insert-function 'magit-commit-add-log-insert
   "Used by `magit-commit-add-log' to insert a single entry.")
@@ -409,9 +415,5 @@ actually insert the entry."
           (insert (format "(%s): \n" defun))
           (backward-char))))))
 
-;;; magit-commit.el ends soon
 (provide 'magit-commit)
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
 ;;; magit-commit.el ends here
