@@ -1,6 +1,6 @@
 ;;; magit-ediff.el --- Ediff extension for Magit  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2016  The Magit Project Contributors
+;; Copyright (C) 2010-2017  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -35,13 +35,13 @@
 (defvar smerge-ediff-buf)
 (defvar smerge-ediff-windows)
 
+;;; Options
+
 (defgroup magit-ediff nil
   "Ediff support for Magit."
+  :link '(info-link "(magit)Ediffing")
   :group 'magit-extensions)
 
-(unless (find-lisp-object-file-name 'magit-ediff-quit-hook 'defvar)
-  (add-hook 'magit-ediff-quit-hook 'magit-ediff-restore-previous-winconf)
-  (add-hook 'magit-ediff-quit-hook 'magit-ediff-cleanup-auxiliary-buffers))
 (defcustom magit-ediff-quit-hook
   '(magit-ediff-cleanup-auxiliary-buffers
     magit-ediff-restore-previous-winconf)
@@ -53,6 +53,7 @@ invoked using Magit."
   :package-version '(magit . "2.2.0")
   :group 'magit-ediff
   :type 'hook
+  :get 'magit-hook-custom-get
   :options '(magit-ediff-cleanup-auxiliary-buffers
              magit-ediff-restore-previous-winconf))
 
@@ -73,7 +74,7 @@ If non-nil, use a third Ediff buffer to distinguish which changes
 in the stash were staged.  In cases where the stash contains no
 staged changes, fall back to a two-buffer Ediff.
 
-More specificaly, a stash is a merge commit, stash@{N}, with
+More specifically, a stash is a merge commit, stash@{N}, with
 potentially three parents.
 
 * stash@{N}^1 represents the HEAD commit at the time the stash
@@ -98,12 +99,13 @@ tree at the time of stashing."
   :group 'magit-ediff
   :type 'boolean)
 
+;;; Commands
+
 (defvar magit-ediff-previous-winconf nil)
 
 ;;;###autoload (autoload 'magit-ediff-popup "magit-ediff" nil t)
 (magit-define-popup magit-ediff-popup
   "Popup console for ediff commands."
-  'magit-diff nil nil
   :actions '((?E "Dwim"          magit-ediff-dwim)
              (?u "Show unstaged" magit-ediff-show-unstaged)
              (?s "Stage"         magit-ediff-stage)
@@ -500,9 +502,5 @@ stash that were staged."
 (defun magit-ediff-restore-previous-winconf ()
   (set-window-configuration magit-ediff-previous-winconf))
 
-;;; magit-ediff.el ends soon
 (provide 'magit-ediff)
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
 ;;; magit-ediff.el ends here
