@@ -1,6 +1,6 @@
 ;;; magit-section.el --- section functionality  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2016  The Magit Project Contributors
+;; Copyright (C) 2010-2017  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -41,10 +41,12 @@
 
 (defgroup magit-section nil
   "Expandable sections."
+  :link '(info-link "(magit)Sections")
   :group 'magit)
 
 (defcustom magit-section-show-child-count t
-  "Whether to append the number of children to section headings."
+  "Whether to append the number of children to section headings.
+This only applies to sections for which doing so makes sense."
   :package-version '(magit . "2.1.0")
   :group 'magit-section
   :type 'boolean)
@@ -270,7 +272,7 @@ If there is no previous sibling section, then move to the parent."
     (set-window-start (selected-window) (magit-section-start section))))
 
 (defun magit-hunk-set-window-start (section)
-  "Ensure the beginning of the `hunk' SECTION is visible.
+  "When SECTION is a `hunk', ensure that its beginning is visible.
 It the SECTION has a different type, then do nothing."
   (when (eq (magit-section-type section) 'hunk)
     (magit-section-set-window-start section)))
@@ -835,7 +837,11 @@ evaluated its BODY.  Admittedly that's a bit of a hack."
 (defvar-local magit-section-unhighlight-sections nil)
 
 (defun magit-section-update-region (_)
-  ;; Don't show complete region.  Highlighting emphasizes headings.
+  "When the region is a valid section-selection, highlight them all."
+  ;; At least that's what it does conceptually.  In actuality it just
+  ;; returns a list of those sections, and it doesn't even matter if
+  ;; this is a member of `magit-region-highlight-hook'.  It probably
+  ;; should be removed, but I want to make sure before removing it.
   (magit-region-sections))
 
 (defun magit-section-update-highlight ()
@@ -862,7 +868,7 @@ evaluated its BODY.  Admittedly that's a bit of a hack."
       (setq deactivate-mark nil))))
 
 (defun magit-section-highlight (section selection)
-  "Highlight SECTION and if non-nil all SELECTION.
+  "Highlight SECTION and if non-nil all sections in SELECTION.
 This function works for any section but produces undesirable
 effects for diff related sections, which by default are
 highlighted using `magit-diff-highlight'.  Return t."
@@ -878,7 +884,7 @@ highlighted using `magit-diff-highlight'.  Return t."
   t)
 
 (defun magit-section-highlight-selection (_ selection)
-  "Highlight the section selection region.
+  "Highlight the section-selection region.
 If SELECTION is non-nil then it is a list of sections selected by
 the region.  The headings of these sections are then highlighted.
 
@@ -1169,9 +1175,5 @@ again use `remove-hook'."
         (set hook value)
       (set-default hook value))))
 
-;;; magit-section.el ends soon
 (provide 'magit-section)
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
 ;;; magit-section.el ends here
